#include <mpi.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>

#define MASTER 0
#define IS_MASTER(x) (x == MASTER ? 1 : 0)

int main(int argc, char *argv[]) {

    enum TAG {
        itineraire,
        bonus
    };

    int numtasks, rank, rc;

    rc = MPI_Init(&argc, &argv);
    if (rc != MPI_SUCCESS) {
        printf("Erreur MPI_Init()");
        MPI_Abort(MPI_COMM_WORLD, rc);
    }

    MPI_Comm_size(MPI_COMM_WORLD, &numtasks);
    MPI_Comm_rank(MPI_COMM_WORLD, &rank);

    if (numtasks == 1) {
        printf("Erreur: deux processus minimum");
        exit(-1);
    }

    if (IS_MASTER(rank)) {
        float *scores = (float*) malloc((numtasks-1)* sizeof(float));
        int *coins = (int*) malloc((numtasks-1)* sizeof(int));

        float max_score = 0;
        int winner_rank = -1;

        do {

            int total_coins = 0;
            int i;

            for (i = 1; i < numtasks; ++i) {

                MPI_Recv(&scores[i - 1], 1, MPI_FLOAT, i, itineraire, MPI_COMM_WORLD, MPI_STATUS_IGNORE);

                MPI_Recv(&coins[i - 1], 1, MPI_INT, i, bonus, MPI_COMM_WORLD, MPI_STATUS_IGNORE);
            }

            for (i = 1; i < numtasks; ++i) {

                /* Calculer le max */
                if (max_score < scores[i - 1]) {
                    max_score = scores[i - 1];
                    winner_rank = i;
                }

                /* Compter les points */
                total_coins += coins[i - 1];
            }

            printf("Momentanément en tête le processus %d avec %f km \n", winner_rank, max_score);
            printf("Points total des joueurs: %d\n\n", total_coins);

        } while(max_score < 100.0);

        printf("Le vainqueur est le processus %d\n", winner_rank);
    }
    else {

        float itineraire_en_km = 0;
        int coins = 0;

        srand(time(NULL)*(rank+1));

        do {

            itineraire_en_km += (rand() % 50) * rank % 30;
            coins += (rand() % 10);

            usleep(100000);

            MPI_Send(&itineraire_en_km, 1, MPI_FLOAT, MASTER, itineraire, MPI_COMM_WORLD);
            MPI_Send(&coins, 1, MPI_INT, MASTER, bonus, MPI_COMM_WORLD);

        } while(itineraire_en_km < 100.0);

    }

    MPI_Finalize();
    return 0;
}