#include <stdio.h>
#include <mpi.h>

#define NUM_DATA_TOT 1024
#define NUM_TASK 4
#define NUM_DATA_PARTIELLE NUM_DATA_TOT / NUM_TASK

int main(int argc, char *argv[]) {
    double data_envoye[NUM_DATA_TOT];
    double data_recu[NUM_DATA_PARTIELLE];
    int numtasks, rank;

    MPI_Init(&argc,&argv);
    MPI_Comm_size(MPI_COMM_WORLD, &numtasks);
    MPI_Comm_rank(MPI_COMM_WORLD, &rank);

    if (NUM_TASK != numtasks) {
        printf("Il faut definir %d processus", NUM_TASK);
        return 0;
    }

    if (rank == 0) {
        /* initialisation des donné à envoyer */
        for (int i = 0; i < NUM_DATA_TOT; ++i)
            data_envoye[i] = 3.14 * i;
    }

    /**
    *  les données sont partagées entre tous les processus,
    *  y compris le processus master avec rank égal à zéro
    **/
    MPI_Scatter(data_envoye, NUM_DATA_PARTIELLE, MPI_DOUBLE,
            data_recu,      NUM_DATA_PARTIELLE, MPI_DOUBLE,
            0, MPI_COMM_WORLD);

    /* le processus courant travaillera seulement sur une portion des donneés */
    double somme_partielle = 0;

    /* procédure d'addition mono-thread */
    for (int i = 0; i < NUM_DATA_PARTIELLE; ++i)
        somme_partielle += data_recu[i];

    double somme_totale;
    MPI_Allreduce(&somme_partielle, &somme_totale, 1, MPI_DOUBLE, MPI_SUM, MPI_COMM_WORLD);

    if (rank == 0)
        printf("somme des tous les elements = %f\n", somme_totale);

    MPI_Finalize();

    return 0;
}
